@@ -19,6 +19,10 @@ typedef struct qr {
     unsigned char *rlens;
     unsigned char  WD;
     unsigned char WDB;
+    unsigned char neccblk1;
+    unsigned char neccblk2;
+    unsigned char datablkw;
+    unsigned char eccblkwid;
 
     unsigned initialized : 1;
 }
@@ -26,6 +30,11 @@ qr_t;
 
 #define QRBIT(f,x,y) ( ( qr->f[((x)>>3) + (y) * qr->WDB] >> (7-((x) & 7 ))) & 1 )
 
+#define QR_MINIMUM_VERSION 1
+#define QR_MAXIMUM_VERSION 40
+#define QR_MINIMUM_LEVEL 1
+#define QR_MAXIMUM_LEVEL 4
+
 #endif /* def HEADER */
 
 #define SETQRBIT(f,x,y) qr->f[((x)>>3) + (y) * qr->WDB] |= 0x80 >> ((x) & 7)
@@ -217,12 +226,6 @@ static void freeframe(qr_t * qr)
     free (qr->strinbuf);
 }
 
-static unsigned char ECCLEVEL;
-static unsigned char neccblk1;
-static unsigned char neccblk2;
-static unsigned char datablkw;
-static unsigned char eccblkwid;
-
 static const unsigned char eccblocks[] = {
    1, 0, 19,  7,   1, 0, 16, 10,   1, 0, 13, 13,   1, 0,  9, 17,
    1, 0, 34, 10,   1, 0, 28, 16,   1, 0, 22, 22,   1, 0, 16, 28,
@@ -268,7 +271,8 @@ static const unsigned char eccblocks[] = {
 
 unsigned initecc(qr_t * qr)
 {
-    assert (qr->version > 0 && qr->version <= 40);
+    assert (qr->version >= QR_MINIMUM_VERSION &&
+	    qr->version <= QR_MAXIMUM_VERSION);
 
     qr->WD = 17 + 4 * qr->version;
     qr->WDB = (qr->WD + 7) / 8;
@@ -281,21 +285,20 @@ unsigned initecc(qr_t * qr)
     qr->qrframe = malloc (fsz);
     assert (qr->qrframe);
 
-    assert (qr->level >= 1 && qr->level <= 4);
-    ECCLEVEL = qr->level;
+    assert (qr->level >= QR_MINIMUM_LEVEL && qr->level <= QR_MAXIMUM_LEVEL);
 
     unsigned eccindex = (qr->level - 1) * 4 + (qr->version - 1) * 16;
 
-    neccblk1 = eccblocks[eccindex++];
-    neccblk2 = eccblocks[eccindex++];
-    datablkw = eccblocks[eccindex++];
-    eccblkwid = eccblocks[eccindex++];
+    qr->neccblk1 = eccblocks[eccindex++];
+    qr->neccblk2 = eccblocks[eccindex++];
+    qr->datablkw = eccblocks[eccindex++];
+    qr->eccblkwid = eccblocks[eccindex++];
 
-    if (fsz < datablkw + (datablkw + eccblkwid) * (neccblk1 + neccblk2) + neccblk2)
-        fsz = datablkw + (datablkw + eccblkwid) * (neccblk1 + neccblk2) + neccblk2;
+    if (fsz < qr->datablkw + (qr->datablkw + qr->eccblkwid) * (qr->neccblk1 + qr->neccblk2) + qr->neccblk2)
+        fsz = qr->datablkw + (qr->datablkw + qr->eccblkwid) * (qr->neccblk1 + qr->neccblk2) + qr->neccblk2;
     qr->strinbuf = calloc(fsz, sizeof (unsigned char));
     qr->initialized = 1;
-    return datablkw * (neccblk1 + neccblk2) + neccblk2 - 3;     //-2 if vers <= 9!
+    return qr->datablkw * (qr->neccblk1 + qr->neccblk2) + qr->neccblk2 - 3;     //-2 if vers <= 9!
 }
 
 /* Initialize using string length. */
@@ -311,10 +314,10 @@ unsigned initeccsize (qr_t * qr)
 
     for (vers = 1; vers < 40; vers++) {
         eccindex = (qr->level - 1) * 4 + (vers - 1) * 16;
-        neccblk1 = eccblocks[eccindex++];
-        neccblk2 = eccblocks[eccindex++];
-        datablkw = eccblocks[eccindex++];
-        if (qr->input_length < datablkw * (neccblk1 + neccblk2) + neccblk2 - 3) {
+        qr->neccblk1 = eccblocks[eccindex++];
+        qr->neccblk2 = eccblocks[eccindex++];
+        qr->datablkw = eccblocks[eccindex++];
+        if (qr->input_length < qr->datablkw * (qr->neccblk1 + qr->neccblk2) + qr->neccblk2 - 3) {
 	    qr->version = vers;
             break;
 	}
@@ -421,7 +424,7 @@ static void stringtoqr (qr_t * qr)
     memcpy (qr->strinbuf, (unsigned char *) qr->input, qr->input_length);
     size = qr->input_length;
 
-    max = datablkw * (neccblk1 + neccblk2) + neccblk2;
+    max = qr->datablkw * (qr->neccblk1 + qr->neccblk2) + qr->neccblk2;
     if (size >= max - 2) {
         size = max - 2;
         if (qr->version > 9)
@@ -458,32 +461,32 @@ static void stringtoqr (qr_t * qr)
     // calculate and append ECC
     unsigned char *ecc = &qr->strinbuf[max];
     unsigned char *dat = qr->strinbuf;
-    initrspoly(eccblkwid,qr->qrframe);
+    initrspoly(qr->eccblkwid,qr->qrframe);
 
-    for (i = 0; i < neccblk1; i++) {
-        appendrs(dat, datablkw, ecc, eccblkwid, qr->qrframe);
-        dat += datablkw;
-        ecc += eccblkwid;
+    for (i = 0; i < qr->neccblk1; i++) {
+        appendrs(dat, qr->datablkw, ecc, qr->eccblkwid, qr->qrframe);
+        dat += qr->datablkw;
+        ecc += qr->eccblkwid;
     }
-    for (i = 0; i < neccblk2; i++) {
-        appendrs(dat, datablkw + 1, ecc, eccblkwid, qr->qrframe);
-        dat += datablkw + 1;
-        ecc += eccblkwid;
+    for (i = 0; i < qr->neccblk2; i++) {
+        appendrs(dat, qr->datablkw + 1, ecc, qr->eccblkwid, qr->qrframe);
+        dat += qr->datablkw + 1;
+        ecc += qr->eccblkwid;
     }
     unsigned j;
     dat = qr->qrframe;
-    for (i = 0; i < datablkw; i++) {
-        for (j = 0; j < neccblk1; j++)
-            *dat++ = qr->strinbuf[i + j * datablkw];
-        for (j = 0; j < neccblk2; j++)
-            *dat++ = qr->strinbuf[(neccblk1 * datablkw) + i + (j * (datablkw + 1))];
+    for (i = 0; i < qr->datablkw; i++) {
+        for (j = 0; j < qr->neccblk1; j++)
+            *dat++ = qr->strinbuf[i + j * qr->datablkw];
+        for (j = 0; j < qr->neccblk2; j++)
+            *dat++ = qr->strinbuf[(qr->neccblk1 * qr->datablkw) + i + (j * (qr->datablkw + 1))];
     }
-    for (j = 0; j < neccblk2; j++)
-        *dat++ = qr->strinbuf[(neccblk1 * datablkw) + i + (j * (datablkw + 1))];
-    for (i = 0; i < eccblkwid; i++)
-        for (j = 0; j < neccblk1 + neccblk2; j++)
-            *dat++ = qr->strinbuf[max + i + j * eccblkwid];
-    memcpy(qr->strinbuf, qr->qrframe, max + eccblkwid * (neccblk1 + neccblk2));
+    for (j = 0; j < qr->neccblk2; j++)
+        *dat++ = qr->strinbuf[(qr->neccblk1 * qr->datablkw) + i + (j * (qr->datablkw + 1))];
+    for (i = 0; i < qr->eccblkwid; i++)
+        for (j = 0; j < qr->neccblk1 + qr->neccblk2; j++)
+            *dat++ = qr->strinbuf[max + i + j * qr->eccblkwid];
+    memcpy(qr->strinbuf, qr->qrframe, max + qr->eccblkwid * (qr->neccblk1 + qr->neccblk2));
 
 }
 
@@ -516,7 +519,7 @@ static void fillframe(qr_t * qr)
     ffgohv = 1;
 
     /* inteleaved data and ecc codes */
-    for (i = 0; i < ((datablkw + eccblkwid) * (neccblk1 + neccblk2) + neccblk2); i++) {
+    for (i = 0; i < ((qr->datablkw + qr->eccblkwid) * (qr->neccblk1 + qr->neccblk2) + qr->neccblk2); i++) {
         d = qr->strinbuf[i];
         for (j = 0; j < 8; j++, d <<= 1) {
             if (0x80 & d)
@@ -747,7 +750,7 @@ static const unsigned fmtword[] = {
 static void addfmt(qr_t * qr, unsigned char masknum)
 {
     unsigned fmtbits;
-    unsigned char i, lvl = ECCLEVEL - 1;
+    unsigned char i, lvl = qr->level - 1;
 
     fmtbits = fmtword[masknum + (lvl << 3)];
     // low byte
